from typing import (
    Any,
    Dict,
    List,
    NoReturn,
    Optional
)
from ._base import (
    DEFAULT,
    TestCaseFamily,
    _BaseGenerator
)
from ..common import format


class Generator(_BaseGenerator):

    _static_write_mode: int = DEFAULT.GENERATOR.INPUT.WRITE_MODE
    _default_write_mode: int = DEFAULT.GENERATOR.INPUT.WRITE_MODE

    def __init__(
        self,
        target_dir: str,
        *test_case_families: TestCaseFamily,
        file_extension: str = DEFAULT.GENERATOR.INPUT.FILE_EXTENSION,
        indexing: bool = DEFAULT.GENERATOR.INPUT.INDEXING,
        skip_errors: bool = DEFAULT.GENERATOR.SKIP_ERRORS,
        file_size_limit: Optional[int] = DEFAULT.GENERATOR.FILE_SIZE_LIMIT
    ):
        super().__init__(target_dir, file_extension, skip_errors, file_size_limit)
        self.__family_pool: List[TestCaseFamily] = [*test_case_families]
        self.__indexing = indexing

    @property
    def family_pool(self): return self.__family_pool
    @property
    def indexing(self): return self.__indexing

    @family_pool.setter
    def family_pool(
        self, value: List[TestCaseFamily]): self.__family_pool = value

    @indexing.setter
    def indexing(self, value: bool): self.__indexing = value

    def add(self, *args: TestCaseFamily | Any) -> NoReturn:
        for obj in args:
            family = obj
            if not isinstance(obj, TestCaseFamily):
                family = TestCaseFamily(obj)
            self.__family_pool.append(family)

    def remove(self, arg: TestCaseFamily | Any) -> NoReturn:
        if isinstance(arg, TestCaseFamily):
            self.__family_pool.remove(arg)
        else:
            for family in self.__family_pool:
                if arg == family.gen_func:
                    self.__family_pool.remove(family)

    @classmethod
    def _create_test_case(
        cls,
        family: TestCaseFamily,
        gen_func_args: List[Any],
        gen_func_kwargs: Dict[Any, Any]
    ) -> str:
        return family.create(gen_func_args, gen_func_kwargs)

    @classmethod
    def _generate(
        cls,
        target_dir: str,
        file_extension: str,
        indexing: bool,
        is_static_gen: bool,
        skip_errors: bool,
        file_size_limit: int,
        *families: TestCaseFamily
    ) -> bool:
        cls._generate_init(is_static_gen)
        target_dir = cls._get_dir(target_dir)
        repository = []
        failed = False
        for family in families:
            for itest in range(family.number_of_test_cases):
                if indexing:
                    test_case_name = str(len(repository))
                else:
                    i_tmp = itest
                    while True:
                        name_tmp = f'{family.name}_{i_tmp}'
                        if name_tmp not in repository:
                            test_case_name = name_tmp
                            break
                        i_tmp += 1
                res = cls._create_file(
                    repository,
                    target_dir,
                    test_case_name + '.' + file_extension,
                    is_static_gen,
                    skip_errors,
                    file_size_limit,
                    family,
                    family.args_list[itest],
                    family.kwargs_list[itest]
                )
                if 'Fail' in res:
                    failed = True
                print(res)
        if failed:
            print(format.formating(
                "All test case's inputs have been generated, except for a few.", format.YELLOW))
            return False
        print(format.formating(
            "All test case's inputs have been generated.", format.GREEN))
        return True

    @classmethod
    def generate(
        cls,
        target_dir: str,
        *families: TestCaseFamily,
        file_extension: str = DEFAULT.GENERATOR.INPUT.FILE_EXTENSION,
        indexing: bool = DEFAULT.GENERATOR.INPUT.INDEXING,
        skip_errors: bool = DEFAULT.GENERATOR.SKIP_ERRORS,
        file_size_limit: Optional[int] = DEFAULT.GENERATOR.FILE_SIZE_LIMIT
    ) -> bool:
        print('\nStart generating input for test cases:')
        return cls._generate(
            target_dir,
            file_extension,
            indexing,
            True,
            skip_errors,
            file_size_limit,
            *families
        )

    def start(self) -> bool:
        print('\nStart generating input for test cases:')
        return self._generate(
            self.target_dir,
            self.file_extension,
            self.__indexing,
            False,
            self.skip_errors,
            self.file_size_limit,
            *self.__family_pool
        )



from random import (
    choices as randchoices,
    randint,
    triangular,
    randrange,
    uniform,
    choice as randchoice,
    shuffle as randshuffle,
    sample,
    random as randrd
)
from typing import Dict, Sequence
from ..exceptions import InvalidArgumentValueException, TypeException

_ONE = 1


class ItRandom:
    def __init__(self):
        pass

    def lucky(self, true_percent=0.5):
        return randchoices((True, False), cum_weights=(true_percent, 1))[0]

    def rdint(self, start, end=None, step=_ONE, /):
        if end is None:
            if step is not _ONE:
                raise TypeException(
                    "Missing a non-None 'end' argument (Thiếu đối số 'end').")
            return randrange(0, start + 1)
        if start > end:
            start, end = end, start
        return randrange(start, end + 1, step)

    def rdfloat(self, start=None, end=None, gather_at=None, /):
        if end is None:
            if start is None:
                if gather_at is not None:
                    raise TypeException(
                        "Missing a non-None 'start' and 'end' arguments (Thiếu 2 đối số 'start', 'end').")
                return randrd()
            if gather_at is not None:
                return triangular(0, start, gather_at)
            return uniform(0, start)
        if start > end:
            start, end = end, start
        if gather_at is not None:
            return triangular(start, end, gather_at)
        return uniform(start, end)

    def choices(self, population, length, probabilities=None):
        if probabilities:
            if isinstance(probabilities, Sequence):
                _probabilities = list(probabilities)
                if len(_probabilities) < len(population):
                    _probabilities.extend(
                        [1] * (len(population) - len(_probabilities)))
                return randchoices(population, weights=_probabilities, k=length)
            if isinstance(probabilities, Dict):
                _population = set(population)
                _population = tuple(_population)
                _probabilities = probabilities.copy()
                for ele in _population:
                    if _probabilities.get(ele):
                        continue
                    _probabilities[ele] = 1
                _probabilities = [_probabilities[ele] for ele in _population]
                return randchoices(population, weights=_probabilities, k=length)
            raise TypeException(
                "'probabilities' must be a tuple/list/dict ('probabilities' phải là tuple/list/dict).")
        return randchoices(population, k=length)

    def choice(self, population, probabilities=None):
        if probabilities:
            return self.choices(population, 1, probabilities)[0]
        return randchoice(population)

    def shuffle(self, sequence):
        randshuffle(sequence)

    def once(self, population, length, probabilities=None):
        if probabilities:
            if isinstance(probabilities, Sequence):
                _population = tuple(_population)
                _probabilities = list(probabilities)
                if len(_probabilities) < len(population):
                    _probabilities.extend(
                        [1] * (len(population) - len(_probabilities)))
            if isinstance(probabilities, Dict):
                _population = set(population)
                _population = tuple(_population)
                _probabilities = probabilities.copy()
                for ele in _population:
                    if _probabilities.get(ele):
                        continue
                    _probabilities[ele] = 1
                _probabilities = [_probabilities[ele] for ele in _population]
                return randchoices(population, weights=_probabilities, k=length)
            raise TypeException(
                "'probabilities' must be a tuple/list/dict ('probabilities' phải là tuple/list/dict).")

        if length > len(_population):
            raise InvalidArgumentValueException(
                "Can't return a list 'length' sized, the maximum length that can be achieved is %d ('length' không được lớn hơn %d)" % (len(_population), len(_population)))
        if probabilities:
            res = []
            idx = tuple(range(len(_population)))
            while len(res) < length:
                i = self.choice(idx, _probabilities)
                _probabilities[i] = 0
                res.append(_population[i])
            return res
        return sample(_population, length)

    def withsum(self, sumvalue, length, *, lowerbound=0, upperbound=None):
        if upperbound is not None:
            if lowerbound >= upperbound:
                raise InvalidArgumentValueException(
                    "'upperbound' cannot be less than or equal to 'lowerbound'('lowerbound' phải nhỏ hơn hoặc bằng 'upperbound').")
            if upperbound * length < sumvalue:
                raise InvalidArgumentValueException(
                    "Can't create a list with sum is 'sumvalue' because 'upperbound' is too small ('upperbound' quá nhỏ để tạo list có tổng bằng 'sumvalue').")
        if lowerbound >= 0:
            if lowerbound * length > sumvalue:
                raise InvalidArgumentValueException(
                    "Can't create a list with sum is 'sumvalue' because 'lowerbound' is too big ('lowerbound' quá lớn để tạo list có tổng bằng 'sumvalue').")
            _res = [lowerbound] * length
            sumvalue -= lowerbound * length
            _ready = list(range(length))
            while sumvalue > 0:
                i = randchoice(_ready)
                if _res[i] == upperbound:
                    _ready.remove(i)
                    continue
                _res[i] += 1
                sumvalue -= 1
        else:
            if upperbound is not None:
                max_of_lowele = (upperbound * length -
                                 sumvalue) / (upperbound - lowerbound)
            else:
                max_of_lowele = length - 1

            lowsize = randint(0, int(max_of_lowele))
            minlow = lowerbound * lowsize
            if upperbound is not None:
                maxhigh = upperbound * (length - lowsize)
                highsum = randint(sumvalue, min(maxhigh, sumvalue - minlow))
            else:
                highsum = randint(sumvalue, sumvalue - minlow)
            lowsum = sumvalue - highsum

            _low = [-1] * lowsize
            lowsum += lowsize
            _ready = list(range(lowsize))
            while lowsum < 0:
                i = randchoice(_ready)
                if _low[i] == lowerbound:
                    _ready.remove(i)
                    continue
                _low[i] -= 1
                lowsum += 1
            _high = [0] * (length - lowsize)
            _ready = list(range(length - lowsize))
            while highsum > 0:
                i = randchoice(_ready)
                if _high[i] == upperbound:
                    _ready.remove(i)
                    continue
                _high[i] += 1
                highsum -= 1
            _res = _low + _high
            shuffle(_res)
        return _res


_inst = ItRandom()
def lucky(*arg, **kwargs): return _inst.lucky(*arg, **kwargs)
def rdint(*arg, **kwargs): return _inst.rdint(*arg, **kwargs)
def rdfloat(*arg, **kwargs): return _inst.rdfloat(*arg, **kwargs)
def choices(*arg, **kwargs): return _inst.choices(*arg, **kwargs)
def choice(*arg, **kwargs): return _inst.choice(*arg, **kwargs)
def shuffle(*arg, **kwargs): return _inst.shuffle(*arg, **kwargs)
def once(*arg, **kwargs): return _inst.once(*arg, **kwargs)


def withsum(*arg, **kwargs):
    try:
        return _inst.withsum(*arg, **kwargs)
    except TypeError as e:
        if 'positional arguments but' in str(e):
            raise TypeException(
                "'lowerbound' and 'upperbound' must be keyword arguments ('lowerbound', 'upperbound' phải được truyền dưới dạng từ khóa).")
        raise e



from typing import Dict, List, overload


def isprime(integer: int) -> bool:
    '''
    Check if `integer` is prime or not.

    Kiểm tra số nguyên tố.
    '''


def factorization(integer: int) -> Dict[int, int]:
    '''
    Integer factorization.

    Phân tích thừa số nguyên tố.
    '''


def divisors(integer: int) -> List[int]:
    '''
    Return a list of integers which is divisors of `integer`.

    Trả về list ước của `integer`.
    '''


@overload
def Eratosthenes(end: int) -> Dict[int, bool]:
    '''
    Sieve of Eratosthenes from 1 to `end`. 

    Sàng nguyên tố Eratosthenes từ 1 cho tới `end`.
    '''


@overload
def Eratosthenes(start: int, end: int) -> Dict[int, bool]:
    '''
    Sieve of Eratosthenes from `start` to `end`. Constraint: |`end` - `start`| ≤ 1_000_000.

    Sàng nguyên tố Eratosthenes từ `start cho tới `end`. Ràng buộc: |`end` - `start`| ≤ 1_000_000.
    '''



from io import (
    BytesIO,
    TextIOWrapper
)
from sys import settrace
from threading import Thread
from time import time
from ..common.exceptions import (
    JudgeException,
    RunTimeError,
    InvalidReturn,
    TimeLimitExceeded
)
from subprocess import run as systemrun
from ..common import (
    format as _format,
    stdout as _stdout
)
from ._common import (
    TestCaseInfo,
    JudgeResult,
    SynAssets
)
from platform import system as osname

class StoppableThread(Thread):
    def __init__(
        self,
        group=None,
        target=None,
        name=None,
        args=(),
        kwargs=None,
        *,
        daemon=None
    ):
        super().__init__(group, target, name, args=args, kwargs=kwargs, daemon=daemon)
        self.exception = None
        self.__killed = False

    def start(self):
        self.__run_backup = self.run
        self.run = self._run
        Thread.start(self)

    def _run(self):
        settrace(self.globaltrace)
        try:
            self.__run_backup()
        except JudgeException as e:
            self.exception = e
        self.run = self.__run_backup

    def globaltrace(self, frame, event, arg):
        if event == 'call':
            return self.localtrace
        return None

    def localtrace(self, frame, event, arg):
        if self.__killed:
            if event == 'line':
                raise SystemExit()
        return self.localtrace

    def kill(self):
        self.__killed = True


class TestCaseRunner(StoppableThread):
    def __init__(
        self,
        command,
        judge_result: JudgeResult
    ):
        super().__init__(daemon=True)
        self.command = command
        self.judge_result = judge_result
        self.exeption = None

    def _run_process(self):
        start_time = time()
        process = systemrun(self.command, stdout=-1)
        end_time = time()
        self.judge_result.cost_time = end_time - start_time
        if process.returncode != 0:
            raise RunTimeError()
        with TextIOWrapper(BytesIO(process.stdout)) as outstream:
            self.judge_result.output = outstream.read()

    def run(self):
        settrace(self.globaltrace)
        try:
            self._run_process()
        except JudgeException as e:
            self.exeption = e


class TestCaseManager(Thread):
    def __init__(
        self,
        judge,
        time_limit,
        solution_path,
        actualoutput_dir,
        testcase_queue: SynAssets,
    ):
        self.judge = judge
        self.stdcursor = self.judge.stdcursor
        self.time_limit = time_limit
        self.solution_path = solution_path
        self.actualoutput_dir = actualoutput_dir
        self.testcase_queue = testcase_queue
        self.judge_result = None
        self.runner = None
        super().__init__()

    def setup(
        self,
        testcase_info: TestCaseInfo
    ):
        self.testcase_info = testcase_info

        self.stdcursor.lock.acquire()
        self.stdcursor.assets.jumpto(
            testcase_info.order, testcase_info.display_start_index)
        _stdout.print(_format.formating('Judging...', _format.YELLOW))
        self.stdcursor.lock.release()

        self.language = self.solution_path[self.solution_path.rfind('.') + 1:]

        if self.language == 'py':
            command = f'python {self.solution_path} < {testcase_info.input_file_path}'
        elif self.language == 'cpp':
            command = f'judgingprocess < {testcase_info.input_file_path}'
        if osname() == 'Windows':
            command = f'cmd /c "{command}'
        if self.judge_result is not None:
            del self.judge_result
        if self.runner is not None:
            del self.runner
        self.judge_result = JudgeResult(testcase_info)
        self.runner = TestCaseRunner(command, self.judge_result)

    def _run_testcase(self):
        self.runner.start()
        if self.time_limit is None:
            self.runner.join()
        else:
            self.runner.join(self.time_limit)

        if self.runner.exeption is not None:
            raise self.runner.exeption
        if (
            self.time_limit is not None
            and (
                self.runner.judge_result.cost_time is None
                or
                self.runner.judge_result.cost_time > self.time_limit
            )
        ):
            self.runner.kill()
            raise TimeLimitExceeded()

        if (
            self.language == 'py' and self.judge_result.output.startswith(
                'Traceback')
            or
            self.language == 'cpp' and 'terminate called after throwing' in self.judge_result.output
        ):
            raise InvalidReturn()

    def run(self):
        while True:
            self.testcase_queue.lock.acquire()
            stop_flag = False
            if self.testcase_queue.assets:
                self.setup(self.testcase_queue.assets.pop())
            else:
                stop_flag = True
            self.testcase_queue.lock.release()
            if stop_flag:
                return
            try:
                self._run_testcase()
            except InvalidReturn:
                self.judge_result.result = 'IR'
            except RunTimeError:
                self.judge_result.result = 'RTE'
            except TimeLimitExceeded:
                self.judge_result.result = 'TLE'
            if self.judge_result.output is not None:
                self._check_result()
                if self.actualoutput_dir is not None:
                    with open(f'{self.actualoutput_dir}/{self.testcase_info.testname}.txt', 'w', encoding='utf-8') as actualoutput_file:
                        actualoutput_file.write(self.judge_result.output)
            self._report_result()

    def _check_result(self):
        with open(self.testcase_info.output_file_path, 'r', encoding='utf-8') as expected_outputfile:
            if self.judge_result.output.strip() == expected_outputfile.read().strip():
                self.judge_result.result = 'AC'
                self.judge.accnt += 1
            else:
                self.judge_result.result = 'WA'

    def _report_result(self):
        self.stdcursor.lock.acquire()
        self.stdcursor.assets.jumpto(
            self.testcase_info.order, self.testcase_info.display_start_index)
        _stdout.print(self.judge_result)
        self.stdcursor.lock.release()



from .judge import (
    SingleJudge,
    MultiJudge
)
from ._common import (
    SynAssets,
    JudgeResult,
    TestCaseInfo
)
from ._thread import (
    StoppableThread,
    TestCaseRunner,
    TestCaseManager
)


from typing import (
    Optional as _Optional,
    Sequence as _Sequence
)
from ._stdout import format as _format

'''
Exceptions that may happen in all the code.
'''


class HlightException(Exception):
    '''
    Base exception.
    '''

    def __init__(self,
                 msg: _Optional[str] = None,
                 screen: _Optional[str] = None,
                 stacktrace: _Optional[_Sequence[str]] = None) -> None:
        if msg is None:
            msg = '...'
        msg = _format.formating(msg, _format.RED)
        self.msg = msg
        self.screen = screen
        self.stacktrace = stacktrace

    def __str__(self) -> str:
        exception_msg = "\n"

        def label(lb: str):
            return _format.formating(" +) " + lb + ": ", _format.BOLD, _format.DARKCYAN)
        exception_msg += label('Message') + "%s\n" % self.msg
        if self.screen:
            exception_msg += label('Screenshot') + "Available via screen\n"
        if self.stacktrace:
            stacktrace = "\n".join(self.stacktrace)
            exception_msg += label('Stacktrace') + "\n%s" % stacktrace
        return exception_msg


class FailedToCreateTestCaseException(HlightException):
    '''
    Thrown when a test case creation process has an error.
    '''

    def __init__(self,
                 filename: str = None,
                 error: _Optional[str] = None,
                 screen: _Optional[str] = None,
                 stacktrace: _Optional[_Sequence[str]] = None) -> None:
        msg = f'Failed to create "{filename}"' + \
            (': ' + error if error else '!')
        super().__init__(msg, screen, stacktrace)


class WrongFileExtensionException(HlightException):
    '''
    Thrown when no files with the set extension exist.
    '''

    def __init__(self,
                 file_ets: _Optional[str] = None,
                 screen: _Optional[str] = None,
                 stacktrace: _Optional[_Sequence[str]] = None) -> None:
        msg = f'No files with the set extension exist (extension: ".{file_ets}").'
        super().__init__(msg, screen, stacktrace)


class TypeException(HlightException):
    '''
    Thrown when actual type is not expected type.
    '''


class InvalidArgumentValueException(HlightException):
    '''
    Thrown when an argument value is invalid.
    '''


class MissingFileException(HlightException):
    '''
    Thrown when the requested file is missing.
    '''


class JudgeException(HlightException):
    '''
    Base exception for judging.
    '''


class CompileError(JudgeException):
    '''
    Thrown when compilation fails.
    '''


class RunTimeError(JudgeException):
    '''
    Thrown when an error occurs during run time.
    '''


class InvalidReturn(JudgeException):
    '''
    Thrown when the running program throws an exception.
    '''


class TimeLimitExceeded(JudgeException):
    '''
    Thrown when program's runtime exceeds the execution time permitted.
    '''



from io import (
    TextIOWrapper,
    BytesIO
)
import sys


class format:
    '''
    Change format for stdout text.
    '''
    PURPLE = '\033[95m'
    CYAN = '\033[96m'
    DARKCYAN = '\033[36m'
    BLUE = '\033[94m'
    GREEN = '\033[92m'
    YELLOW = '\033[93m'
    RED = '\033[91m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'
    END = '\033[0m'
    GRAY = '\033[90m'

    @staticmethod
    def formating(text: str, *fm: str) -> str:
        '''
        Return a string has been formated with the given formatting.
        '''
        return ''.join([*fm]) + text + format.END

    @staticmethod
    def color(r: int, g: int, b: int) -> str:
        '''
        Return a string which is ANSI code of a color.
        '''
        return "\033[38;2;{};{};{}m{} \033[38;2;255;255;255m".format(r, g, b)


class cursor:
    '''
    Control the stdout cursor.
    '''
    @staticmethod
    def up(n: int = 1):
        '''
        Move up n lines.
        '''
        stdout.print(f'\033[{n}A')

    @staticmethod
    def down(n: int = 1):
        '''
        Move down n lines.
        '''
        stdout.print(f'\033[{n}B')

    @staticmethod
    def next(n: int = 1):
        '''
        Move next n chars.
        '''
        stdout.print(f'\033[{n}C')

    @staticmethod
    def back(n: int = 1):
        '''
        Move back n chars.
        '''
        stdout.print(f'\033[{n}D')

    @staticmethod
    def move_to(x, y):
        stdout.print(f'\033[{x};{y}H')

    @staticmethod
    def clear_lines(n: int = 0):
        '''
        Clear n lines.
        '''
        stdout.print(f'\r\033[K')
        for i in range(n):
            cursor.up()
            stdout.print(f'\r\033[K')

    @staticmethod
    def del_line(left_to_right=True):
        '''
        Delete chars from left to right.
        '''
        if left_to_right:
            stdout.print('\033[0K')
        else:
            stdout.print('\033[1K')


class stdout:
    @staticmethod
    def print(*args):
        print(*args, end='')

    @staticmethod
    def get(func, *args, **kwargs) -> str:
        '''
        Get stdout's content of a function call.
        '''
        old_stdout = sys.stdout
        sys.stdout = TextIOWrapper(BytesIO(), sys.stdout.encoding)
        wrapper = sys.stdout
        func(*args, **kwargs)
        wrapper.seek(0)
        output = wrapper.read()
        sys.stdout = old_stdout
        wrapper.close()
        return output


from typing import Optional
from ..common.exceptions import (
    CompileError,
    WrongFileExtensionException
)
from ._base import (
    _BaseGenerator,
    DEFAULT
)
from os import (
    listdir,
    popen,
    remove,
    system
)
from platform import system as osname
from ..common import format


class Generator(_BaseGenerator):
    '''
    ### Test case's output generator.
    '''

    _static_write_mode: int = DEFAULT.GENERATOR.OUTPUT.WRITE_MODE
    _default_write_mode: int = DEFAULT.GENERATOR.OUTPUT.WRITE_MODE

    def __init__(
        self,
        input_testsuite_dir: str,
        solution_path: str,
        target_dir: str,
        *,
        input_file_extension: str = DEFAULT.GENERATOR.OUTPUT.INPUT_FILE_EXTENSION,
        file_extension: str = DEFAULT.GENERATOR.OUTPUT.FILE_EXTENSION,
        skip_errors: bool = DEFAULT.GENERATOR.SKIP_ERRORS,
        file_size_limit: Optional[int] = DEFAULT.GENERATOR.FILE_SIZE_LIMIT
    ):
        super().__init__(target_dir, file_extension, skip_errors, file_size_limit)
        self.__solution_path = solution_path
        self.__input_testsuite_dir = input_testsuite_dir
        self.__input_file_extension = input_file_extension

    @property
    def input_testsuite_dir(self) -> str: return self.__input_testsuite_dir
    @property
    def solution_path(self) -> str: return self.__solution_path
    @property
    def input_file_extension(self) -> str: return self.__input_file_extension

    @input_testsuite_dir.setter
    def input_testsuite_dir(
        self, value) -> None: self.__input_testsuite_dir = value

    @solution_path.setter
    def solution_path(self, value) -> None: self.__solution_path = value

    @input_file_extension.setter
    def input_file_extension(
        self, value) -> None: self.input_file_extension = value

    @classmethod
    def _create_test_case(
        cls,
        input_path: str,
        solution_path: str,
        language: str
    ) -> str:
        if language == 'py':
            command = 'python ' + solution_path + ' < ' + input_path
        elif language == 'cpp':
            command = 'solution < ' + input_path
        if osname() == 'Windows':
            command = f'cmd /c "{command}'
        with popen(command) as output:
            return output.read()

    @classmethod
    def _generate(
        cls,
        source_dir: str,
        solution_path: str,
        target_dir: str,
        input_file_extension: str,
        file_extension: str,
        is_static_gen: bool,
        skip_errors: bool,
        file_size_limit: int
    ) -> bool:
        cls._generate_init(is_static_gen)
        source_dir = cls._get_dir(source_dir)
        target_dir = cls._get_dir(target_dir)
        repository = []
        failed = False
        wrong_file_extension = True
        language = solution_path[solution_path.rfind('.') + 1:]
        if language == 'cpp':
            if system('g++ ./' + solution_path + ' -o ./solution'):
                raise CompileError(f'Failed to compile "{solution_path}".')
        warm_up = False
        for file in listdir(source_dir):
            if not file.endswith('.' + input_file_extension):
                continue
            if not warm_up:
                # Prepare step to warm up the generator - does not cause errors in time when creating tests.
                for i in range(2):
                    cls._create_test_case(
                        source_dir + '/' + file,
                        solution_path,
                        language
                    )
                warm_up = True
            wrong_file_extension = False
            test_case_name = file[:file.rfind('.')]
            res = cls._create_file(
                repository,
                target_dir,
                test_case_name + '.' + file_extension,
                is_static_gen,
                skip_errors,
                file_size_limit,
                source_dir + '/' + file,
                solution_path,
                language
            )
            if 'Fail' in res:
                failed = True
            print(res)
        if language == 'cpp':
            if osname() == 'Linux':
                remove('solution')
            elif osname() == 'Darwin':
                remove('solution')
            elif osname() == 'Windows':
                remove('solution.exe')
        if wrong_file_extension:
            raise WrongFileExtensionException(input_file_extension)
        if failed:
            print(format.formating(
                "All test case's outputs have been generated, except for a few.", format.YELLOW))
            return False
        print(format.formating(
            "All test case's outputs have been generated.", format.GREEN))
        return True

    @classmethod
    def generate(
        cls,
        input_testsuite_dir: str,
        solution_path: str,
        target_dir: str,
        *,
        input_file_extension: str = DEFAULT.GENERATOR.OUTPUT.INPUT_FILE_EXTENSION,
        file_extension: str = DEFAULT.GENERATOR.OUTPUT.FILE_EXTENSION,
        skip_errors: bool = DEFAULT.GENERATOR.SKIP_ERRORS,
        file_size_limit: Optional[int] = DEFAULT.GENERATOR.FILE_SIZE_LIMIT
    ) -> bool:
        print('\nStart generating output for test cases:')
        return cls._generate(
            input_testsuite_dir,
            solution_path,
            target_dir,
            input_file_extension,
            file_extension,
            True,
            skip_errors,
            file_size_limit
        )

    def start(self) -> bool:
        print('\nStart generating output for test cases:')
        return self._generate(
            self.__input_testsuite_dir,
            self.__solution_path,
            self.target_dir,
            self.__input_file_extension,
            self.file_extension,
            False,
            self.skip_errors,
            self.file_size_limit
        )



from ..common import (
    format as _format,
    cursor as _cursor,
    stdout as _stdout
)
from threading import Lock


class Cursor:
    def __init__(self, cur_line):
        self.cur_line = cur_line

    def jumpto(self, dest, start_index):
        delta = dest - self.cur_line
        self.cur_line = dest
        if delta > 0:
            _cursor.down(delta)
        elif delta < 0:
            _cursor.up(-delta)
        _stdout.print('\r')
        _cursor.next(start_index)
        _cursor.del_line()


class SynAssets:
    def __init__(self, assets):
        self.lock = Lock()
        self.assets = assets


class TestCaseInfo:
    def __init__(self, order, testname, input_file_path, output_file_path):
        self.order = order
        self.testname = testname
        self.input_file_path = input_file_path
        self.output_file_path = output_file_path
        self.display_start_index = None


class JudgeResult:
    def __init__(self, testcase_info: TestCaseInfo, result=None, cost_time=None, output=None):
        self.testcase_info = testcase_info
        self.result = result
        self.cost_time = cost_time
        self.output = output

    def get(self):
        return self.result == 'AC'

    def __str__(self):
        if self.result == 'WA':
            return _format.formating('Wrong answer (WA).', _format.RED)
        if self.result == 'IR':
            return _format.formating('Invalid Return (IR).', _format.RED)
        if self.result == 'RTE':
            return _format.formating('Run Time Error (RTE).', _format.RED)
        if self.result == 'TLE':
            return _format.formating('Time Limit Exceeded (TLE).', _format.RED)
        if self.result == 'AC':
            return _format.formating('Accepted (AC) in %.3fs.' % self.cost_time, _format.GREEN)
        return _format.formating('Unknown.', _format.YELLOW)



from io import (
    BytesIO,
    TextIOWrapper
)
from os import (
    listdir,
    makedirs,
    remove,
    system,
    cpu_count
)
from os.path import exists
from time import time
from ..common.exceptions import (
    MissingFileException,
    RunTimeError,
    CompileError,
    InvalidReturn,
    TimeLimitExceeded
)
from subprocess import (
    run as systemrun,
    PIPE,
    DEVNULL
)
from ..common import (
    format as _format
)

from ._thread import (
    StoppableThread,
    TestCaseManager
)

from ._common import (
    JudgeResult,
    SynAssets,
    TestCaseInfo,
    Cursor
)
from platform import system as osname

class _DEFAULT:
    TIME_LIMIT = None
    INPUT_EXTENSION = 'in'
    OUTPUT_EXTENSION = 'out'
    ACTUALOUTPUT_DIRECTORY = None
    JUDGE_IN_TURN = False
    try:
        NUMBER_OF_JUDGES = cpu_count()
    except:
        NUMBER_OF_JUDGES = 4


class _BaseJudge:
    def __init__(
        self,
        intestsuite_dir,
        outtestsuite_dir,
        solution_path,
        *,
        time_limit=_DEFAULT.TIME_LIMIT,
        input_extension=_DEFAULT.INPUT_EXTENSION,
        output_extension=_DEFAULT.OUTPUT_EXTENSION,
        actualout_dir=_DEFAULT.ACTUALOUTPUT_DIRECTORY
    ) -> None:

        self.intestsuite_dir = intestsuite_dir
        self.outtestsuite_dir = outtestsuite_dir
        self.solution_path = solution_path
        self.time_limit = time_limit
        self.input_extension = input_extension
        self.output_extension = output_extension
        self.actualout_dir = actualout_dir
        self.language = solution_path[solution_path.rfind('.') + 1:]
        # Compile
        if self.language == 'cpp':
            if system('g++ ./' + solution_path + ' -o ./judgingprocess'):
                raise CompileError(f'Failed to compile "{solution_path}".')

        if actualout_dir is not None and not exists(actualout_dir):
            makedirs(actualout_dir)

        self.testcnt = 0
        self.testsuite = SynAssets(list[TestCaseInfo]())
        for filename in listdir(intestsuite_dir):
            if not filename.endswith('.' + input_extension):
                continue
            testname = filename[:filename.rfind('.')]
            if not exists(outtestsuite_dir + '/' + testname + '.' + output_extension):
                raise MissingFileException(
                    'Mising output file for test case "%s" (Thiếu file output).' % testname)
            self.testsuite.assets.append(
                TestCaseInfo(
                    order=self.testcnt,
                    testname=testname,
                    input_file_path=intestsuite_dir + '/' + filename,
                    output_file_path=outtestsuite_dir + '/' + testname + '.' + output_extension
                )
            )
            self.testcnt += 1

    def start(self) -> None:
        if self.accnt == 0:
            result = _format.formating(
                f'Failed ({self.accnt}/{self.testcnt}).', _format.BOLD, _format.RED)
        elif self.accnt == self.testcnt:
            result = _format.formating(
                f'Accepted ({self.accnt}/{self.testcnt}).', _format.BOLD, _format.GREEN)
        else:
            result = _format.formating(
                f'Partial ({self.accnt}/{self.testcnt}).', _format.BOLD, _format.YELLOW)

        print(_format.formating(f'  > Result: ',
              _format.BOLD, _format.CYAN) + result)

        if self.language == 'cpp':
            if osname() == 'Linux':
                systemrun('killall -KILL judgingprocess', stdout=DEVNULL, stderr=DEVNULL)
                remove('judgingprocess')
            elif osname() == 'Darwin':
                systemrun('sudo killall -9 \'judgingprocess\'', stdout=DEVNULL, stderr=DEVNULL)
                remove('judgingprocess')
            elif osname() == 'Windows':
                systemrun('taskkill /F /IM judgingprocess.exe', stdout=DEVNULL, stderr=DEVNULL)
                remove('judgingprocess.exe')


class SingleJudge(_BaseJudge):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)

    def __run_testcase(
        self,
        testcase_info: TestCaseInfo,
        actual_output_file_path=None
    ):
        if self.language == 'py':
            command = f'python {self.solution_path} < {testcase_info.input_file_path}'
        elif self.language == 'cpp':
            command = f'judgingprocess < {testcase_info.input_file_path}'
        if osname() == 'Windows':
            command = f'cmd /c "{command}'
        self.judge_result = JudgeResult(testcase_info)

        run_thread = StoppableThread(target=self.__visual_run, args=(command,))
        run_thread.start()
        run_thread.join(self.time_limit)
        if run_thread.exception is not None:
            raise run_thread.exception
        if run_thread.is_alive():
            run_thread.kill()
            raise TimeLimitExceeded()
        if (
            self.language == 'py' and self.judge_result.output.startswith(
                'Traceback')
            or
            self.language == 'cpp' and 'terminate called after throwing' in self.judge_result.output
        ):
            raise InvalidReturn()

        if actual_output_file_path is not None:
            with open(actual_output_file_path, 'w', encoding='utf-8') as actual_outputfile:
                actual_outputfile.write(self.judge_result.output)
        with open(testcase_info.output_file_path, 'r', encoding='utf-8') as expected_outputfile:
            if self.judge_result.output.strip() == expected_outputfile.read().strip():
                self.judge_result.result = 'AC'
            else:
                self.judge_result.result = 'WA'

    def __visual_run(self, command):
        start_time = time()
        process = systemrun(command, stdout=PIPE)
        end_time = time()
        self.judge_result.cost_time = end_time - start_time
        if process.returncode != 0:
            raise RunTimeError()
        with TextIOWrapper(BytesIO(process.stdout)) as outstream:
            output = outstream.read()
            self.judge_result.output = output

    def start(self) -> None:
        print('\nStart judgment:')
        self.accnt = 0
        for i, testcase_info in enumerate(self.testsuite.assets):
            actual_output_file_path = None
            if self.actualout_dir is not None:
                actual_output_file_path = self.actualout_dir + \
                    '/' + testcase_info.testname + '.txt'
            try:
                self.__run_testcase(testcase_info, actual_output_file_path)
                if self.judge_result.result == 'WA':
                    status = _format.formating(
                        'Wrong answer (WA).', _format.RED)
                elif self.judge_result.result == 'AC':
                    status = _format.formating(
                        'Accepted (AC) in %.3fs.' % self.judge_result.cost_time, _format.GREEN)
                    self.accnt += 1
            except InvalidReturn:
                status = _format.formating('Invalid Return (IR).', _format.RED)
            except RunTimeError:
                status = _format.formating(
                    'Run Time Error (RTE).', _format.RED)
            except TimeLimitExceeded:
                status = _format.formating(
                    'Time Limit Exceeded (TLE).', _format.RED)
            print(
                f'- Testcase {i}."{_format.formating(testcase_info.testname, _format.CYAN)}": {status}')
        super().start()


class MultiJudge(_BaseJudge):
    def __init__(self, *args, number_of_judges=_DEFAULT.NUMBER_OF_JUDGES, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.number_of_judges = number_of_judges

    def start(self) -> None:
        print('\nStart judgment:')
        self.accnt = 0

        for i in range(len(self.testsuite.assets)):
            string = f'- Testcase {i}."{_format.formating(self.testsuite.assets[i].testname, _format.CYAN)}": Waiting...'
            self.testsuite.assets[i].display_start_index = string.find(
                'Waiting...') - len(_format.CYAN) - len(_format.END)
            print(string)

        self.stdcursor = SynAssets(Cursor(self.testcnt))
        self.stdcursor.lock.acquire()
        self.stdcursor.lock.release()
        self.testsuite.assets = self.testsuite.assets[::-1]

        judges = [TestCaseManager(self, self.time_limit, self.solution_path,
                                  self.testsuite) for _ in range(self.number_of_judges)]

        for judge in judges:
            judge.start()

        for judge in judges:
            judge.join()

        self.stdcursor.lock.acquire()
        self.stdcursor.assets.jumpto(self.testcnt, 0)
        self.stdcursor.lock.release()

        super().start()



from typing import (
    Dict,
    List,
    MutableSequence,
    Sequence,
    TypeVar,
    overload
)

_T = TypeVar("_T")


class ItRandom:
    '''
    Random number generator class.
    '''

    def __init__(self) -> None: ...

    # lucky()
    @overload
    def lucky(self) -> bool:
        '''
        Random choice between True and False with ratio 1:1.

        ## Vietsub

        Lựa chọn ngẫu nhiên giữa True và False với tỉ lệ 1:1.
        '''
    @overload
    def lucky(self, true_percent: float) -> bool:
        '''
        Random choice between True and False with percentage returns True of `true_percent`. Constraint: 0 ≤ `true_percent` ≤ 1.

        ## Vietsub

        Lựa chọn ngẫu nhiên giữa True và False với tỉ lệ phần trăm trả về True là `true_percent`. Ràng buộc: 0 ≤ `true_percent` ≤ 1.
        '''

    # rdint()
    @overload
    def rdint(self, end: int, /) -> int:
        '''
        Return a random integer in range [0, `end`].

        ## Vietsub

        Trả về ngẫu nhiên một số nguyên trong đoạn [0, `end`].
        '''
    @overload
    def rdint(self, start: int, end: int, /) -> int:
        '''
        Return a random integer in range [`start`, `end`].

        ## Vietsub

        Trả về ngẫu nhiên một số nguyên trong đoạn [`start`, `end`].
        '''
    @overload
    def rdint(self, start: int, end: int, step: int, /) -> int:
        '''
        Return a random integer in range [`start`, `end`] with step is `step`.

        ## Vietsub

        Trả về ngẫu nhiên một số nguyên trong đoạn [`start`, `end`] với bước nhảy là `step`.

        -> Random in `range(start, end + 1, step)`.
        '''

    # rdfloat()
    @overload
    def rdfloat(self) -> float:
        '''
        Return a random float in [0, 1].

        ## Vietsub

        Trả về ngẫu nhiên một số thực trong đoạn [0, 1].
        '''
    @overload
    def rdfloat(self, end: float, /) -> float:
        '''
        Return a random float in [0, `end`].

        ## Vietsub

        Trả về ngẫu nhiên một số thực trong đoạn [0, `end`].
        '''
    @overload
    def rdfloat(self, start: float, end: float, /) -> float:
        '''
        Return a random float in [`start`, `end`].

        ## Vietsub

        Trả về ngẫu nhiên một số thực trong đoạn [`start`, `end`].
        '''
    @overload
    def rdfloat(self, start: float, end: float, gather_at: float, /) -> float:
        '''
        Return a random float in [`start`, `end`] but tends to be close to `gather_at`.

        ## Vietsub

        Trả về ngẫu nhiên một số thực trong đoạn [0, 1] nhưng có xu hướng gần với giá trị `gather_at`.
        '''

    # choices()
    @overload
    def choices(self, population: Sequence[_T], length: int) -> List[_T]:
        '''
        Return a `length` sized list of `population` elements chosen with replacement.

        ## Vietsub

        Trả về một list có `length` phần tử. Các phần tử trong list này được chọn ngẫu nhiên từ `population`.
        '''
    @overload
    def choices(self, population: Sequence[_T], length: int, probabilities: Sequence[int]) -> List[_T]:
        '''
        Return a `length` sized list of `population` elements chosen with replacement.

        The probability of selection of each element is based on `probabilities`, map by element index.

        The probability at the missing indexes will be 1.

        ## Vietsub

        Trả về một list có `length` phần tử. Các phần tử trong list này được chọn ngẫu nhiên từ `population`.

        Xác suất lựa chọn của mỗi phần tử được dựa trên `probabilities`, ánh xạ bởi vị trí các phần tử.

        Xác suất tại các chỉ mục bị khuyết sẽ là 1.

        ## Example

        For each element in `choices([1, 1, 2, 3, 4, 5], 3, [6, 1, 0])`:
            - 70% choose 1.
            - 0% choose 2.
            - 10% choose 3.
            - 10% choose 4.
            - 10% choose 5.
        '''
    @overload
    def choices(self, population: Sequence[_T], length: int, probabilities: Dict[_T, int]) -> List[_T]:
        '''
        Return a `length` sized list of `population` elements chosen with replacement.

        The probability of selection of each element is based on `probabilities`, map by element value.

        The probability for missing value will be 1.

        ## Vietsub

        Trả về một list có `length` phần tử. Các phần tử trong list này được chọn ngẫu nhiên từ `population`.

        Xác suất lựa chọn của mỗi phần tử được dựa trên `probabilities`, ánh xạ bởi giá trị các phần tử.

        Xác suất cho các giá trị bị khuyết sẽ là 1.

        ## Example

        For each element in `choices([1, 1, 2, 3, 4, 5], 3, {0:100, 1:7, 2:0})`:
            - 70% choose 1.
            - 0% choose 2.
            - 10% choose 3.
            - 10% choose 4.
            - 10% choose 5.
        '''

    # choice()
    @overload
    def choice(self, population: Sequence[_T]) -> _T:
        '''
        Return a random element from `population`.

        ## Vietsub

        Trả về một phần tử ngẫu nhiên của `population`.
        '''
    @overload
    def choice(self, population: Sequence[_T], probabilities: Sequence[int]) -> _T:
        '''
        Return a random element from `population` with probabilities is `probabilities`, map by element index.

        The probability at the missing indexes will be 1.

        ## Vietsub

        Trả về một phần tử ngẫu nhiên của `population` với xác suất lựa chọn là `probabilities`, ánh xạ bởi vị trí các phần tử.

        Xác suất tại các chỉ mục bị khuyết sẽ là 1.

        ## Example

        Return value of `choice([1, 1, 2, 3, 4, 5], [6, 1, 0])`:
            - 70% is 1.
            - 0% is 2.
            - 10% is 3.
            - 10% is 4.
            - 10% is 5.
        '''
    @overload
    def choice(self, population: Sequence[_T], probabilities: Dict[_T, int]) -> _T:
        ''' 
        Return random element from `population` with probabilities is `probabilities`, map by element value.

        The probability for missing value will be 1.

        ## Vietsub

        Trả về một phần tử ngẫu nhiên của `population` với xác suất lựa chọn là `probabilities`, ánh xạ bởi giá trị các phần tử.

        Xác suất cho các giá trị bị khuyết sẽ là 1.

        ## Example

        Return value of `choice([1, 1, 2, 3, 4, 5], {0:100, 1:7, 2:0})`:
            - 70% is 1.
            - 0% is 2.
            - 10% is 3.
            - 10% is 4.
            - 10% is 5.
        '''

    # shuffle()
    def shuffle(self, sequence: MutableSequence[_T]) -> None:
        '''
        Shuffle the sequence.

        ## Vietsub

        Xáo trộn dãy.
        '''

    # once()
    @overload
    def once(self, population: Sequence[_T], length: int) -> List[_T]:
        '''
        Return a `length` sized list of `population` elements chosen with replacement, but each element appearing only once.

        ## Vietsub

        Trả về một list có `length` phần tử. Các phần tử trong list này được chọn ngẫu nhiên từ `population`, mỗi phần tử chỉ xuất hiện 1 lần.
        '''
    @overload
    def once(self, population: Sequence[_T], length: int, probabilities: Sequence[int]) -> List[_T]:
        '''
        Return a `length` sized list of `population` elements chosen with replacement, but each element appearing only once.

        The probability of selection of each element is based on `probabilities`, map by element index.

        The probability at the missing indexes will be 1.

        ## Vietsub

        Trả về một list có `length` phần tử. Các phần tử trong list này được chọn ngẫu nhiên từ `population`, mỗi phần tử chỉ xuất hiện 1 lần.

        Xác suất lựa chọn của mỗi phần tử được dựa trên `probabilities`, ánh xạ bởi vị trí các phần tử.

        Xác suất tại các chỉ mục bị khuyết sẽ là 1.
        '''
    @overload
    def once(self, population: Sequence[_T], length: int, probabilities: Dict[_T, int]) -> List[_T]:
        '''
        Return a `length` sized list of `population` elements chosen with replacement, but each element appearing only once.

        The probability of selection of each element is based on `probabilities`, map by element value.

        The probability for missing value will be 1.

        ## Vietsub

        Trả về một list có `length` phần tử. Các phần tử trong list này được chọn ngẫu nhiên từ `population`, mỗi phần tử chỉ xuất hiện 1 lần.

        Xác suất lựa chọn của mỗi phần tử được dựa trên `probabilities`, ánh xạ bởi giá trị các phần tử.

        Xác suất cho các giá trị bị khuyết sẽ là 1.
        '''

    # withsum()
    @overload
    def withsum(self, sumvalue: int, length: int) -> List[int]:
        '''
        Returns a integer list with `length` elements whose sum is `sumvalue`.

        Lower bound is 0.

        ## Vietsub

        Trả về một list số nguyên với `length` phần tử có tổng là `sumvalue`.

        Cận dưới là 0.
        '''
    @overload
    def withsum(self, sumvalue: int, length: int, *, upperbound: int) -> List[int]:
        '''
        Returns a integer list with `length` elements whose sum is `sumvalue`.

        Lower bound is 0.

        Upper bound is `upperbound`.

        ## Vietsub

        Trả về một list số nguyên với `length` phần tử có tổng là `sumvalue`.

        Cận dưới là 0.

        Cận trên là `upperbound`.
        '''
    @overload
    def withsum(self, sumvalue: int, length: int, *, lowerbound: int, upperbound: int) -> List[int]:
        '''
        Returns a integer list with `length` elements whose sum is `sumvalue`.

        Lower bound is `lowerbound`.

        Upper bound is `upperbound`.

        ## Vietsub

        Trả về một list số nguyên với `length` phần tử có tổng là `sumvalue`.

        Cận dưới là `lowerbound`.

        Cận trên là `upperbound`.
        '''

# lucky()


@overload
def lucky() -> bool:
    '''
    Random choice between True and False with ratio 1:1.

    ## Vietsub

    Lựa chọn ngẫu nhiên giữa True và False với tỉ lệ 1:1.
    '''


@overload
def lucky(true_percent: float) -> bool:
    '''
    Random choice between True and False with percentage returns True of `true_percent`. Constraint: 0 ≤ `true_percent` ≤ 1.

    ## Vietsub

    Lựa chọn ngẫu nhiên giữa True và False với tỉ lệ phần trăm trả về True là `true_percent`. Ràng buộc: 0 ≤ `true_percent` ≤ 1.
    '''

# rdint()


@overload
def rdint(end: int, /) -> int:
    '''
    Return a random integer in range [0, `end`].

    ## Vietsub

    Trả về ngẫu nhiên một số nguyên trong đoạn [0, `end`].
    '''


@overload
def rdint(start: int, end: int, /) -> int:
    '''
    Return a random integer in range [`start`, `end`].

    ## Vietsub

    Trả về ngẫu nhiên một số nguyên trong đoạn [`start`, `end`].
    '''


@overload
def rdint(start: int, end: int, step: int, /) -> int:
    '''
    Return a random integer in range [`start`, `end`] with step is `step`.

    ## Vietsub

    Trả về ngẫu nhiên một số nguyên trong đoạn [`start`, `end`] với bước nhảy là `step`.

    -> Random in `range(start, end + 1, step)`.
    '''

# rdfloat()


@overload
def rdfloat(self) -> float:
    '''
    Return a random float in [0, 1].

    ## Vietsub

    Trả về ngẫu nhiên một số thực trong đoạn [0, 1].
    '''


@overload
def rdfloat(end: float, /) -> float:
    '''
    Return a random float in [0, `end`].

    ## Vietsub

    Trả về ngẫu nhiên một số thực trong đoạn [0, `end`].
    '''


@overload
def rdfloat(start: float, end: float, /) -> float:
    '''
    Return a random float in [`start`, `end`].

    ## Vietsub

    Trả về ngẫu nhiên một số thực trong đoạn [`start`, `end`].
    '''


@overload
def rdfloat(start: float, end: float, gather_at: float, /) -> float:
    '''
    Return a random float in [`start`, `end`] but tends to be close to `gather_at`.

    ## Vietsub

    Trả về ngẫu nhiên một số thực trong đoạn [0, 1] nhưng có xu hướng gần với giá trị `gather_at`.
    '''

# choices()


@overload
def choices(population: Sequence[_T], length: int) -> List[_T]:
    '''
    Return a `length` sized list of `population` elements chosen with replacement.

    ## Vietsub

    Trả về một list có `length` phần tử. Các phần tử trong list này được chọn ngẫu nhiên từ `population`.
    '''


@overload
def choices(population: Sequence[_T], length: int, probabilities: Sequence[int]) -> List[_T]:
    '''
    Return a `length` sized list of `population` elements chosen with replacement.

    The probability of selection of each element is based on `probabilities`, map by element index.

    The probability at the missing indexes will be 1.

    ## Vietsub

    Trả về một list có `length` phần tử. Các phần tử trong list này được chọn ngẫu nhiên từ `population`.

    Xác suất lựa chọn của mỗi phần tử được dựa trên `probabilities`, ánh xạ bởi vị trí các phần tử.

    Xác suất tại các chỉ mục bị khuyết sẽ là 1.

    ## Example

    For each element in `choices([1, 1, 2, 3, 4, 5], 3, [6, 1, 0])`:
        - 70% choose 1.
        - 0% choose 2.
        - 10% choose 3.
        - 10% choose 4.
        - 10% choose 5.
    '''


@overload
def choices(population: Sequence[_T], length: int, probabilities: Dict[_T, int]) -> List[_T]:
    '''
    Return a `length` sized list of `population` elements chosen with replacement.

    The probability of selection of each element is based on `probabilities`, map by element value.

    The probability for missing value will be 1.

    ## Vietsub

    Trả về một list có `length` phần tử. Các phần tử trong list này được chọn ngẫu nhiên từ `population`.

    Xác suất lựa chọn của mỗi phần tử được dựa trên `probabilities`, ánh xạ bởi giá trị các phần tử.

    Xác suất cho các giá trị bị khuyết sẽ là 1.

    ## Example

    For each element in `choices([1, 1, 2, 3, 4, 5], 3, {0:100, 1:7, 2:0})`:
        - 70% choose 1.
        - 0% choose 2.
        - 10% choose 3.
        - 10% choose 4.
        - 10% choose 5.
    '''

# choice()


@overload
def choice(population: Sequence[_T]) -> _T:
    '''
    Return a random element from `population`.

    ## Vietsub

    Trả về một phần tử ngẫu nhiên của `population`.
    '''


@overload
def choice(population: Sequence[_T], probabilities: Sequence[int]) -> _T:
    '''
    Return a random element from `population` with probabilities is `probabilities`, map by element index.

    The probability at the missing indexes will be 1.

    ## Vietsub

    Trả về một phần tử ngẫu nhiên của `population` với xác suất lựa chọn là `probabilities`, ánh xạ bởi vị trí các phần tử.

    Xác suất tại các chỉ mục bị khuyết sẽ là 1.

    ## Example

    Return value of `choice([1, 1, 2, 3, 4, 5], [6, 1, 0])`:
        - 70% is 1.
        - 0% is 2.
        - 10% is 3.
        - 10% is 4.
        - 10% is 5.
    '''


@overload
def choice(population: Sequence[_T], probabilities: Dict[_T, int]) -> _T:
    ''' 
    Return random element from `population` with probabilities is `probabilities`, map by element value.

    The probability for missing value will be 1.

    ## Vietsub

    Trả về một phần tử ngẫu nhiên của `population` với xác suất lựa chọn là `probabilities`, ánh xạ bởi giá trị các phần tử.

    Xác suất cho các giá trị bị khuyết sẽ là 1.

    ## Example

    Return value of `choice([1, 1, 2, 3, 4, 5], {0:100, 1:7, 2:0})`:
        - 70% is 1.
        - 0% is 2.
        - 10% is 3.
        - 10% is 4.
        - 10% is 5.
    '''

# shuffle()


def shuffle(sequence: MutableSequence[_T]) -> None:
    '''
    Shuffle the sequence.

    ## Vietsub

    Xáo trộn dãy.
    '''

# once()


@overload
def once(population: Sequence[_T], length: int) -> List[_T]:
    '''
    Return a `length` sized list of `population` elements chosen with replacement, but each element appearing only once.

    ## Vietsub

    Trả về một list có `length` phần tử. Các phần tử trong list này được chọn ngẫu nhiên từ `population`, mỗi phần tử chỉ xuất hiện 1 lần.
    '''


@overload
def once(population: Sequence[_T], length: int, probabilities: Sequence[int]) -> List[_T]:
    '''
    Return a `length` sized list of `population` elements chosen with replacement, but each element appearing only once.

    The probability of selection of each element is based on `probabilities`, map by element index.

    The probability at the missing indexes will be 1.

    ## Vietsub

    Trả về một list có `length` phần tử. Các phần tử trong list này được chọn ngẫu nhiên từ `population`, mỗi phần tử chỉ xuất hiện 1 lần.

    Xác suất lựa chọn của mỗi phần tử được dựa trên `probabilities`, ánh xạ bởi vị trí các phần tử.

    Xác suất tại các chỉ mục bị khuyết sẽ là 1.
    '''


@overload
def once(population: Sequence[_T], length: int, probabilities: Dict[_T, int]) -> List[_T]:
    '''
    Return a `length` sized list of `population` elements chosen with replacement, but each element appearing only once.

    The probability of selection of each element is based on `probabilities`, map by element value.

    The probability for missing value will be 1.

    ## Vietsub

    Trả về một list có `length` phần tử. Các phần tử trong list này được chọn ngẫu nhiên từ `population`, mỗi phần tử chỉ xuất hiện 1 lần.

    Xác suất lựa chọn của mỗi phần tử được dựa trên `probabilities`, ánh xạ bởi giá trị các phần tử.

    Xác suất cho các giá trị bị khuyết sẽ là 1.
    '''

# withsum()


@overload
def withsum(sumvalue: int, length: int) -> List[int]:
    '''
    Returns a integer list with `length` elements whose sum is `sumvalue`.

    Lower bound is 0.

    ## Vietsub

    Trả về một list số nguyên với `length` phần tử có tổng là `sumvalue`.

    Cận dưới là 0.
    '''


@overload
def withsum(sumvalue: int, length: int, *, upperbound: int) -> List[int]:
    '''
    Returns a integer list with `length` elements whose sum is `sumvalue`.

    Lower bound is 0.

    Upper bound is `upperbound`.

    ## Vietsub

    Trả về một list số nguyên với `length` phần tử có tổng là `sumvalue`.

    Cận dưới là 0.

    Cận trên là `upperbound`.
    '''


@overload
def withsum(sumvalue: int, length: int, *, lowerbound: int, upperbound: int) -> List[int]:
    '''
    Returns a integer list with `length` elements whose sum is `sumvalue`.

    Lower bound is `lowerbound`.

    Upper bound is `upperbound`.

    ## Vietsub

    Trả về một list số nguyên với `length` phần tử có tổng là `sumvalue`.

    Cận dưới là `lowerbound`.

    Cận trên là `upperbound`.
    '''



from setuptools import setup

VERSION = '1.0.1'
AUTHOR = 'Hlight'
DESCRIPTION = 'Python library of IT Club PTIT HN'

setup(
    name='itptit',
    version=VERSION,
    url='https://github.com/HlighT-Git/itptit',
    license='MIT',
    author=AUTHOR,
    author_email='hlight.mail@gmail.com',
    description=DESCRIPTION,
    package_dir={
        'itptit':'itptit',
        'itptit.common': 'itptit/common',
        'itptit.generator': 'itptit/generator',
        'itptit.judge': 'itptit/judge',
        'itptit.common.math': 'itptit/common/math',
        'itptit.common.random': 'itptit/common/random'
    },
    packages=[
        'itptit',
        'itptit.common',
        'itptit.generator',
        'itptit.judge',
        'itptit.common.math',
        'itptit.common.random'
    ],
    package_data={
        'itptit.common': ['*.pyi'],
        'itptit.generator': ['*.pyi'],
        'itptit.judge': ['*.pyi'],
        'itptit.common.math': ['*.pyi'],
        'itptit.common.random': ['*.pyi']
    },
    include_package_data = True,
    long_description=open('README.md', 'r', encoding='utf-8').read(),
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent"
    ]
)



class SingleJudge:
    '''
    #### Judge the test cases in turn, suitable for simple programs.

    ## Vietsub

    #### Chấm các test case lần lượt, thích hợp cho chương trình đơn giản. 
    '''

    def __init__(
        self,
        intestsuite_dir: str,
        outtestsuite_dir: str,
        solution_path: str,
        *,
        time_limit: float = ...,
        input_extension: str = ...,
        output_extension: str = ...,
        actualout_dir: str = ...
    ) -> None:
        '''
        ### SingleJudge constructor.

        ## Parameters

        #### Position/keyword arguments

            - intestsuite_dir: str

                The directory where input of all test cases located.

                Đường dẫn chứa tất cả các file input của các test case.

            - outtestsuite_dir: str

                The directory where output of all test cases located.

                Đường dẫn chứa tất cả các file output của các test case.

            - solution_path: str

                The path of the judgment target.

                Đường dẫn tới lời giải cần chấm.

        #### Keyword-only arguments

            - time_limit: float (Default: `None`)

                Time limit by seconds for each test case.

                Giới hạn thời gian bằng giây cho mỗi test case. 

            - input_extension: str (Default: `'in'`)

                The file extension of test case's input files.

                Phần mở rộng (đuôi file) các file input của các test case.

            - output_extension: str (Default: `'out'`)

                The file extension of test case's output files

                Phần mở rộng (đuôi file) các file output của các test case.

            - actualout_dir: str (Default: `None`)

                The path contains the actual outputs for each test case that the solution being graded will output.

                Đường dẫn chứa các output thực cho mỗi test case mà lời giải đang chấm sẽ xuất ra.
        '''

    def start(self) -> None:
        '''
        ### Start judgment
        '''


class MultiJudge:
    '''
    #### Judge the test cases with multithreading, judge multiple tests at the same time, suitable for time-consuming programs.

    ## Vietsub

    #### Dùng đa luồng để chấm các test case, chấm nhiều test case cùng lúc, thích hợp cho chương trình chạy tốn thời gian.
    '''

    def __init__(
        self,
        intestsuite_dir: str,
        outtestsuite_dir: str,
        solution_path: str,
        *,
        number_of_judges: int = ...,
        time_limit: float = ...,
        input_extension: str = ...,
        output_extension: str = ...,
        actualout_dir: str = ...
    ) -> None:
        '''
        ### MultiJudge constructor.

        ## Parameters

        #### Position/keyword arguments

            - intestsuite_dir: str

                The directory where input of all test cases located.

                Đường dẫn chứa tất cả các file input của các test case.

            - outtestsuite_dir: str

                The directory where output of all test cases located.

                Đường dẫn chứa tất cả các file output của các test case.

            - solution_path: str

                The path of the judgment target.

                Đường dẫn tới lời giải cần chấm.

        #### Keyword-only arguments

            - number_of_judges: int (Default: `os.cpu_count()` or `4`)

                Number of judges.

                Số lượng máy chấm.

            - time_limit: float (Default: `None`)

                Time limit by seconds for each test case.

                Giới hạn thời gian bằng giây cho mỗi test case. 

            - input_extension: str (Default: `'in'`)

                The file extension of test case's input files.

                Phần mở rộng (đuôi file) các file input của các test case.

            - output_extension: str (Default: `'out'`)

                The file extension of test case's output files

                Phần mở rộng (đuôi file) các file output của các test case.

            - actualout_dir: str (Default: `None`)

                The path contains the actual outputs for each test case that the solution being graded will output.

                Đường dẫn chứa các output thực cho mỗi test case mà lời giải đang chấm sẽ xuất ra.
        '''

    def start(self) -> None:
        '''
        ### Start judgment
        '''



from typing import (
    Any,
    List,
    NoReturn,
    Optional
)
from ._base import (
    TestCaseFamily,
    _BaseGenerator
)


class Generator(_BaseGenerator):
    '''
    ### Test case's input generator.
    '''

    _static_write_mode: int = ...
    _default_write_mode: int = ...

    def __init__(
        self,
        target_dir: str,
        *test_case_families: TestCaseFamily,
        file_extension: str = ...,
        indexing: bool = ...,
        skip_errors: bool = ...,
        file_size_limit: Optional[int] = ...
    ):
        '''
        ### Input Generator constructor.

        ## Parameters

        #### Position/keyword arguments

        - target_dir: str

            The directory where the generator put generated testcases in.

            Đường dẫn chứa các test case sẽ sinh ra.

        - *test_case_families: TestCaseFamily

            All the TestCaseFamily that you want the generator to generate.

            Tất cả các TestCaseFamily sử dụng để sinh các test case.

        #### Keyword-only arguments

        - file_extension: str (Default: `'in'`)

            File extension of result files.

            Phần mở rộng (đuôi file) của các test case sinh ra.

        - indexing: bool (Default: `True`)

            Decide whether the file name containing the test case is indexed or not.

            Đánh số theo thứ tự cho từng file test case sinh ra (nếu không thì sinh theo tên của họ test case).

        - skip_errors: bool (Default: `False`)

            Skip (auto-catch) all exceptions encountered during process.

            Bỏ qua các ngoại lệ xảy ra trong quá trình sinh test.

        - file_size_limit: int (Default: `None`)

            Size limit of each test case file that will be generated (in bytes).

            Giới hạn dung lượng file cho mỗi file test case sinh ra.
        '''

    @property
    def target_dir(self) -> str: ...
    @property
    def file_extension(self) -> str: ...
    @property
    def write_mode(self) -> int: ...
    @property
    def skip_errors(self) -> bool: ...
    @property
    def file_size_limit(self) -> int: ...
    @property
    def family_pool(self) -> List[TestCaseFamily]: ...
    @property
    def indexing(self) -> bool: ...
    @target_dir.setter
    def target_dir(self, value: str) -> None:  ...
    @file_extension.setter
    def file_extension(self, value: str) -> None: ...
    @write_mode.setter
    def write_mode(self, value: int) -> None: ...
    @skip_errors.setter
    def skip_errors(self, value: bool) -> None: ...
    @file_size_limit.setter
    def file_size_limit(self, value: int) -> None: ...
    @family_pool.setter
    def family_pool(self, value: List[TestCaseFamily]) -> None: ...
    @indexing.setter
    def indexing(self, value: bool) -> None: self.__indexing = value

    def add(self, *args: TestCaseFamily | Any) -> NoReturn:
        '''
        Add more TestCaseFamily or generate function to family pool.

        Thêm TestCaseFamily hoặc hàm sinh cho bộ sinh test.

        ## Parameters

        *args: TestCaseFamily | function

            If it is a function, it will be init with TestCaseFamily.

            Nếu đối số truyền vào là một function, nó sẽ được khởi tạo với 1 họ test case.

        Ex with foo():

        add(foo) == add(TestCaseFamily(foo))
        '''

    def remove(self, arg: TestCaseFamily | Any) -> NoReturn:
        '''
        Remove TestCaseFamily `arg` or TestCaseFamily has generate function is `arg` in pool.

        Xóa TestCaseFamily `arg` trong `family_pool` nếu `arg` là một TestCaseFamily.

        Xóa TestCaseFamily có hàm sinh là `arg` trong `family_pool` nếu `arg` là một function.

        ## Parameters

        arg: TestCaseFamily | function.
        '''

    @classmethod
    def generate(
        cls,
        target_dir: str,
        *families: TestCaseFamily,
        file_extension: str = ...,
        indexing: bool = ...,
        skip_errors: bool = ...,
        file_size_limit: Optional[int] = ...
    ) -> bool:
        '''
        ### Static generate.

        ## Parameters

        #### Position/keyword arguments

        - target_dir: str

            The directory where the generator put generated testcases in.

            Đường dẫn chứa các test case sinh ra.

        - *families: TestCaseFamily

            All the TestCaseFamily use to generate test cases.

            Tất cả các TestCaseFamily sử dụng để sinh các test case.

        #### Keyword-only arguments

        - file_extension: str (Default: `'in'`)

            File extension of result files.

            Phần mở rộng (đuôi file) của các test case sinh ra.

        - indexing: bool (Default: `True`)

            Decide whether the file name containing the test case is indexed or not.

            Đánh số theo thứ tự cho từng file test case sinh ra (nếu không thì sinh theo tên của họ test case).

        - skip_errors: bool (Default: `False`)

            Skip (auto-catch) all exceptions encountered during process.

            Bỏ qua các ngoại lệ xảy ra trong quá trình sinh test.

        - file_size_limit: int (Default: `None`)

            Size limit of each test case file that will be generated (in bytes).

            Giới hạn dung lượng file cho mỗi file test case sinh ra.

        ## Returns

        True if generated cleanly - no failed creation, else False.

        True nếu tất cả các test case được sinh ra thành công.
        '''

    def start(self) -> bool:
        '''
        ### Run the generator.

        ## Returns

        True if generated cleanly - no failed creation, else False.

        True nếu tất cả các test case được sinh ra thành công.
        '''



from abc import abstractmethod, ABC
from os.path import exists
from os import makedirs
from time import time
from typing import Any, Dict, List
from ..common.exceptions import FailedToCreateTestCaseException
from ..common import (
    cursor as _cursor,
    format as _format,
    stdout as _stdout
)


class DEFAULT:
    class TESTCASEFAMILY:
        NUMBER_OF_TESTS = 1
        ARGS_LIST = []
        KWARGS_LIST = []
        NAME = 'AnonymousTest'

    class GENERATOR:
        SKIP_ERRORS = False
        FILE_SIZE_LIMIT = None

        class INPUT:
            WRITE_MODE = 0
            FILE_EXTENSION = 'in'
            INDEXING = True

        class OUTPUT:
            WRITE_MODE = 2
            INPUT_FILE_EXTENSION = 'in'
            FILE_EXTENSION = 'out'


class TestCaseFamily:
    '''
    Family of test cases that share the same generator function.

    Họ test case sử dụng chung hàm sinh input.
    '''

    def __init__(
            self,
            gen_func,
            number_of_test_cases: int = DEFAULT.TESTCASEFAMILY.NUMBER_OF_TESTS,
            args_list: List[List[Any]] = DEFAULT.TESTCASEFAMILY.ARGS_LIST,
            kwargs_list: List[Dict[Any, Any]
                              ] = DEFAULT.TESTCASEFAMILY.KWARGS_LIST,
            name: str = DEFAULT.TESTCASEFAMILY.NAME):
        '''
        ### TestCaseFamily constructor.

        ## Parameters

        - gen_func: function

            Generate function which is the function use to generate test cases.

            Hàm sinh nội dung cho input của test case.

        - number_of_test_cases: int (Default: `1`)

            The number of test cases that need to be created using this TestCaseFamily.

            Số lượng test case sẽ sinh ra từ họ này.

        - args_list: List[List[Any]] (Default: `[]`)

            A list contains the position arguments for each test case.

            List chứa các list con. Mỗi list con chứa các đối số theo vị trí sử dụng cho hàm sinh trong mỗi test case.

        - kwargs_list: List[Dict[Any, Any]] (Default: `[]`)

            A list contains the keyword arguments for each test case.

            List chứa các list con. Mỗi list con chứa các đối số theo từ khóa sử dụng cho hàm sinh trong mỗi test case.

        - name: str (Default: `'AnonymousTest'`)

            Name for the TestCaseFamily.

            Tên cho họ test case.
        '''
        self.__gen_func = gen_func
        self.__number_of_test_cases = number_of_test_cases
        if len(args_list) != number_of_test_cases:
            args_list += [[]
                          for _ in range(number_of_test_cases - len(args_list))]
        if len(kwargs_list) != number_of_test_cases:
            kwargs_list += [{}
                            for _ in range(number_of_test_cases - len(kwargs_list))]
        self.__args_list = args_list
        self.__kwargs_list = kwargs_list
        self.__name = name

    @property
    def gen_func(self): return self.__gen_func
    @property
    def number_of_test_cases(self) -> int: return self.__number_of_test_cases
    @property
    def args_list(self) -> List[List]: return self.__args_list
    @property
    def kwargs_list(self) -> List[Dict]: return self.__kwargs_list
    @property
    def name(self) -> str: return self.__name

    @number_of_test_cases.setter
    def number_of_test_cases(self, value: int) -> None:
        self.__number_of_test_cases = value

    @args_list.setter
    def args_list(self, value: List[List]) -> None:
        self.__args_list = value

    @kwargs_list.setter
    def kwargs_list(self, value: List[Dict]) -> None:
        self.__kwargs_list = value

    @name.setter
    def name(self, value: str) -> None:
        self.__name = value

    def create(self, gen_func_args: List[Any] = [], gen_func_kwargs: Dict[Any, Any] = {}) -> str:
        '''
        ### Create input for a test case.

        ## Parameters

        - gen_func_args: List[Any] (Default: `[]`)

            Position arguments for generate function.

            Đối số theo vị trí cho hàm sinh.

        - gen_func_kwargs: Dict[Any, Any] (Default: `{}`)

            Keyword arguments for generate function.

            Đối số theo từ khóa cho hàm sinh.

        ## Returns

        A string which is content of a test case created from generate function of itself.

        Nội dung input cho một test case sinh ra từ họ test case này.
        '''
        return _stdout.get(self.__gen_func, *gen_func_args, **gen_func_kwargs)


class _BaseGenerator(ABC):
    '''
    # DON'T USE THIS!
    '''
    _static_write_mode: int = ...
    _default_write_mode: int = ...

    def __init__(
        self,
        target_dir: str,
        file_extension: str,
        skip_errors: bool,
        file_size_limit: int
    ):
        self.__target_dir = self._get_dir(target_dir)
        self.__file_extension = file_extension
        self.__write_mode = self._default_write_mode
        self.__skip_errors = skip_errors
        self.__file_size_limit = file_size_limit

    @property
    def target_dir(self) -> str: return self.__target_dir
    @property
    def file_extension(self) -> str: return self.__file_extension
    @property
    def write_mode(self) -> int: return self.__write_mode
    @property
    def skip_errors(self) -> bool: return self.__skip_errors
    @property
    def file_size_limit(self) -> int: return self.__file_size_limit
    @target_dir.setter
    def target_dir(self, value: str) -> None:  self.__target_dir = value
    @file_extension.setter
    def file_extension(self, value: str) -> None: self.file_extension = value
    @write_mode.setter
    def write_mode(self, value: int) -> None: self.__write_mode = value
    @skip_errors.setter
    def skip_errors(self, value: bool) -> None: self.skip_errors = value
    @file_size_limit.setter
    def file_size_limit(self, value: int) -> None: self.file_size_limit = value

    @classmethod
    def _get_dir(cls, dir: str) -> str:
        while dir[-1] == '/':
            dir = dir[:-1]
        if not exists(dir):
            makedirs(dir)
        return dir

    @classmethod
    def _overwrite_ask(cls, filename: str, write_mode) -> int:
        if write_mode == 2:
            return 2
        print('-' * 50)
        print(_format.formating(
            f'"{filename}" already exists!', _format.YELLOW))
        print(_format.formating('1: ', _format.GREEN) + "Overwrite.")
        print(_format.formating('2: ', _format.GREEN) + "Overwrite for all.")
        print(_format.formating('Default: ', _format.CYAN) + "Skip.")
        choice = input("You want to: ")
        _cursor.clear_lines(6)
        if choice in ('1', '2'):
            return int(choice)
        return 0

    @classmethod
    def _create_file(
        cls,
        repository: List[str],
        path: str,
        filename: str,
        is_static_gen: bool,
        skip_error: bool,
        file_size_limit: int,
        *args,
        **kwargs
    ) -> str:
        status_choices = ('Skip', 'Overwrite', 'Overwrite')
        status_color = {
            'New': _format.GREEN,
            'Skip': _format.GRAY,
            'Overwrite': _format.CYAN,
            'Fail': _format.RED
        }
        status = 'New'
        if exists(path + '/' + filename):
            if filename not in repository:
                if is_static_gen:
                    cls._static_write_mode = cls._overwrite_ask(
                        filename, cls._static_write_mode)
                    status = status_choices[cls._static_write_mode]
                else:
                    cls.write_mode = cls._overwrite_ask(
                        filename, cls.write_mode)
                    status = status_choices[cls.write_mode]
        repository.append(filename)
        if status == 'Skip':
            return '- ' + _format.formating(f'Skip: "{filename}".', status_color['Skip'])
        with open(path + '/' + filename, 'w', encoding='utf-8') as f:
            print(_format.formating('Creating...', _format.YELLOW))
            start_time = time()
            content = cls._create_test_case(*args, **kwargs)
            end_time = time()
            _cursor.clear_lines(1)
            try:
                if content.strip() == '':
                    raise FailedToCreateTestCaseException(
                        filename, 'This file has no content (File trống).')
                file_size = len(content) + content.count('\n')
                if file_size_limit and file_size >= file_size_limit:
                    raise FailedToCreateTestCaseException(
                        filename, f'This file exceeded the set capacity limit (Expected: {file_size_limit}b, actual: {file_size}b) (Vượt quá giới hạn dung lượng đã thiết lập).')
            except FailedToCreateTestCaseException as e:
                if skip_error:
                    return '- ' + _format.formating(f'Fail: {e.msg}', status_color['Fail'])
                raise e
            f.write(content)
            return '- ' + _format.formating(status, status_color[status]) + ': Created file "%s" successfully in %.5fs!' % (filename, end_time - start_time)

    @classmethod
    def _generate_init(cls, is_static_gen: bool):
        if is_static_gen:
            cls._static_write_mode = cls._default_write_mode
        else:
            cls.write_mode = cls._default_write_mode

    @classmethod
    @abstractmethod
    def _generate(cls, *args, **kwargs): ...
    @classmethod
    @abstractmethod
    def _create_test_case(cls, *args, **kwargs): ...
    @classmethod
    @abstractmethod
    def generate(cls, *args, **kwargs): ...

    @abstractmethod
    def start(self, *args, **kwargs): ...



from typing import (
    List,
    Optional
)
from ._base import (
    TestCaseFamily,
    _BaseGenerator
)


class Generator(_BaseGenerator):
    '''
    ### Test case's output generator.
    '''

    _static_write_mode: int = ...
    _default_write_mode: int = ...

    def __init__(
        self,
        input_testsuite_dir: str,
        solution_path: str,
        target_dir: str,
        *,
        input_file_extension: str = ...,
        file_extension: str = ...,
        skip_errors: bool = ...,
        file_size_limit: Optional[int] = ...
    ):
        '''
        ### Output Generator constructor.

        ## Parameters

        #### Position/keyword arguments

        - input_testsuite_dir: str

            The directory where input of all test cases located.

            Đường dẫn chứa tất cả các file input của các test case.

        - solution_path: str

            The path of the solution.

            Đường dẫn tới lời giải.

        - target_dir: str

            The directory where the generator put generated testcases in.

            Đường dẫn chứa các test case sẽ sinh ra.

        #### Keyword-only arguments

        - input_file_extension: str (Default: `'in'`)

            The file extension of input files.

            Phần mở rộng (đuôi file) của các file input.

        - file_extension: str (Default: `'out'`)

            File extension of result files.

            Phần mở rộng (đuôi file) của các test case sinh ra.

        - skip_errors: bool (Default: `False`)

            Skip (auto-catch) all exceptions encountered during process.

            Bỏ qua các ngoại lệ xảy ra trong quá trình sinh test.

        - file_size_limit: int (Default: `None`)

            Size limit of each test case file that will be generated (in bytes).

            Giới hạn dung lượng file cho mỗi file test case sinh ra.
        '''

    @property
    def target_dir(self) -> str: ...
    @property
    def file_extension(self) -> str: ...
    @property
    def write_mode(self) -> int: ...
    @property
    def skip_errors(self) -> bool: ...
    @property
    def file_size_limit(self) -> int: ...
    @property
    def family_pool(self) -> List[TestCaseFamily]: ...
    @property
    def input_testsuite_dir(self) -> str: ...
    @property
    def solution_path(self) -> str: ...
    @property
    def input_file_extension(self) -> str: ...
    @target_dir.setter
    def target_dir(self, value: str) -> None:  ...
    @file_extension.setter
    def file_extension(self, value: str) -> None: ...
    @write_mode.setter
    def write_mode(self, value: int) -> None: ...
    @skip_errors.setter
    def skip_errors(self, value: bool) -> None: ...
    @file_size_limit.setter
    def file_size_limit(self, value: int) -> None: ...
    @family_pool.setter
    def family_pool(self, value: List[TestCaseFamily]) -> None: ...
    @input_testsuite_dir.setter
    def input_testsuite_dir(self, value) -> None: ...
    @solution_path.setter
    def solution_path(self, value) -> None: ...
    @input_file_extension.setter
    def input_file_extension(self, value) -> None: ...

    @classmethod
    def generate(
        cls,
        input_testsuite_dir: str,
        solution_path: str,
        target_dir: str,
        *,
        input_file_extension: str = ...,
        file_extension: str = ...,
        skip_errors: bool = ...,
        file_size_limit: Optional[int] = ...
    ) -> bool:
        '''
        ### Static generate.

        ## Parameters

        #### Position/keyword arguments

        - input_testsuite_dir: str

            The directory where input of all test cases located.

            Đường dẫn chứa tất cả các file input của các test case.

        - solution_path: str

            The path of the solution.

            Đường dẫn tới lời giải.

        - target_dir: str

            The directory where the generator put generated testcases in.

            Đường dẫn chứa các test case sẽ sinh ra.

        #### Keyword-only arguments

        - input_file_extension: str (Default: `'in'`)

            The file extension of input files

            Phần mở rộng (đuôi file) của các file input.

        - file_extension: str (Default: `'out'`)

            File extension of result files.

            Phần mở rộng (đuôi file) của các test case sinh ra.

        - skip_errors: bool (Default: `False`)

            Skip (auto-catch) all exceptions encountered during process.

            Bỏ qua các ngoại lệ xảy ra trong quá trình sinh test.

        - file_size_limit: int (Default: `None`)

            Size limit of each test case file that will be generated (in bytes).

            Giới hạn dung lượng file cho mỗi file test case sinh ra.

        ## Returns

        True if generated cleanly - no failed creation, else False.

        True nếu tất cả các test case được sinh ra thành công.
        '''

    def start(self) -> bool:
        '''
        ### Run the generator.

        ## Returns

        True if generated cleanly - no failed creation, else False.

        True nếu tất cả các test case được sinh ra thành công.
        '''



from math import sqrt
from ..exceptions import InvalidArgumentValueException


def isprime(integer):
    if (
        integer == 2 or
        integer == 3
    ):
        return True
    if (
        integer < 2 or
        integer % 2 == 0 or
        integer % 3 == 0
    ):
        return False

    sqrtval = int(sqrt(integer))
    for i in range(5, sqrtval + 1, 6):
        if (
            integer % i == 0 or
            integer % (i + 2) == 0
        ):
            return False
    return True


def factorization(integer):
    def _compact(base):
        nonlocal integer
        res = 0
        while integer % base == 0:
            res += 1
            integer //= base
        return res
    _prime_divs = {}
    _2 = _compact(2)
    if _2:
        _prime_divs[2] = _2
    _3 = _compact(3)
    if _3:
        _prime_divs[3] = _3

    sqrtval = int(sqrt(integer))
    for i in range(5, sqrtval + 1, 6):
        _i = _compact(i)
        if _i:
            _prime_divs[i] = _i

    if integer > 0:
        _prime_divs[integer] = 1
    return _prime_divs


def divisors(integer):
    _head = []
    _revtail = []

    sqrtval = int(sqrt(integer))
    for i in range(2, sqrtval + 1):
        if integer % i == 0:
            _head.append(i)
            _revtail.append(integer // i)

    if _head[-1] == _revtail[-1]:
        _revtail.pop()

    return _head + _revtail[::-1]


def Eratosthenes(start, end=None):
    if end is not None:
        if end - start > 1_000_000:
            raise InvalidArgumentValueException(
                '|end - start| > 1,000,000 (hiệu 2 đối số này không được vượt quá 1,000,000).')
        _res = {i: True for i in range(start, end + 1)}
        end_sqrt = int(sqrt(end))
        for i in range(2, end_sqrt + 1):
            j = max(i*i, (start + i - 1) / i*i)
            while j <= end:
                _res[j - start] = False
                j += i
        if 1 >= start:
            _res[1 - start] = False
        return _res
    end = start
    _res = {i: True for i in range(end + 1)}
    _res[0] = False
    _res[1] = False
    for i in range(4, end + 1, 2):
        _res[i] = False
    for i in range(9, end + 1, 3):
        _res[i] = False

    end_sqrt = int(sqrt(end))
    for base in range(5, end_sqrt + 1, 6):
        if _res[base]:
            for i in range(base * base, end + 1, base):
                _res[i] = False
        if _res[base + 2]:
            for i in range((base + 2) * (base + 2), end + 1, base + 2):
                _res[i] = False

    return _res
